@@ -31,6 +31,7 @@ import ctypes
 import json
 import multiprocessing
 import time
+from functools import partial
 from os.path import splitext
 from typing import List
 from typing import Union
@@ -68,8 +69,6 @@ from sonusai.utils import seconds_to_hms
 from sonusai.utils import to_numpy_array
 from sonusai.utils import trim_docstring
 
-genft_global = dict()
-
 
 # noinspection PyGlobalUndefined
 
@@ -110,18 +109,10 @@ def genft(mixdb: dict,
           progress: tqdm = None,
           initial_i_sample_offset: int = 0,
           initial_i_frame_offset: int = 0,
-          initial_o_frame_offset: int = 0,
-          audio_loaded: bool = False) -> (np.ndarray, np.ndarray, dict):
-    mixdb_out, total_samples, transform_frames, feature_frames = init(mixdb=mixdb,
-                                                                      mixid=mixid,
-                                                                      logging=logging)
-
-    genft_global['mixdb'] = mixdb_out
-    genft_global['feature_frames'] = feature_frames
-    genft_global['compute_segsnr'] = compute_segsnr
-    if not audio_loaded:
-        genft_global['noise_audios'] = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
-        genft_global['target_audios'] = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+          initial_o_frame_offset: int = 0) -> (np.ndarray, np.ndarray, dict):
+    mixdb_out, total_samples, transform_frames, feature_frames = init(mixdb=mixdb, mixid=mixid, logging=logging)
+    target_audios = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+    noise_audios = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
 
     fg = FeatureGenerator(frame_size=mixdb_out['frame_size'],
                           feature_mode=mixdb_out['feature'],
@@ -129,15 +120,17 @@ def genft(mixdb: dict,
                           truth_mutex=mixdb_out['truth_mutex'])
     mp_feature = multiprocessing.Array(ctypes.c_float, feature_frames * fg.stride * fg.num_bands)
     mp_truth_f = multiprocessing.Array(ctypes.c_float, feature_frames * fg.num_classes)
-    mp_segsnr = multiprocessing.Array(ctypes.c_float, 0)
     if compute_segsnr:
         mp_segsnr = multiprocessing.Array(ctypes.c_float, transform_frames)
+    else:
+        mp_segsnr = multiprocessing.Array(ctypes.c_float, 0)
 
     feature = to_numpy_array(mp_feature, dtype=np.single).reshape((feature_frames, fg.stride, fg.num_bands))
     truth_f = to_numpy_array(mp_truth_f, dtype=np.single).reshape((feature_frames, fg.num_classes))
-    segsnr = np.empty(0, dtype=np.single)
     if compute_segsnr:
         segsnr = to_numpy_array(mp_segsnr, dtype=np.single)
+    else:
+        segsnr = np.empty(0, dtype=np.single)
 
     # First pass to get offsets
     set_mixture_offsets(mixdb=mixdb_out,
@@ -151,7 +144,12 @@ def genft(mixdb: dict,
         progress = tqdm(total=len(mixdb_out['mixtures']), desc='genft', disable=not show_progress)
         progress_needs_close = True
 
-    p_map(_process_mixture,
+    p_map(partial(_process_mixture,
+                  mixdb=mixdb_out,
+                  feature_frames=feature_frames,
+                  target_audios=target_audios,
+                  noise_audios=noise_audios,
+                  compute_segsnr=compute_segsnr),
           list(range(len(mixdb_out['mixtures']))),
           progress=progress,
           initializer=mp_init,
@@ -174,26 +172,31 @@ def genft(mixdb: dict,
     return feature, truth_f, segsnr, mixdb_out
 
 
-def _process_mixture(mixid: int) -> None:
-    fft = ForwardTransform(N=genft_global['mixdb']['frame_size'] * 4,
-                           R=genft_global['mixdb']['frame_size'])
+def _process_mixture(mixid: int,
+                     mixdb: dict,
+                     feature_frames: int,
+                     target_audios: list,
+                     noise_audios: list,
+                     compute_segsnr: bool) -> None:
+    fft = ForwardTransform(N=mixdb['frame_size'] * 4,
+                           R=mixdb['frame_size'])
 
-    fg = FeatureGenerator(frame_size=genft_global['mixdb']['frame_size'],
-                          feature_mode=genft_global['mixdb']['feature'],
-                          num_classes=genft_global['mixdb']['num_classes'],
-                          truth_mutex=genft_global['mixdb']['truth_mutex'])
+    fg = FeatureGenerator(frame_size=mixdb['frame_size'],
+                          feature_mode=mixdb['feature'],
+                          num_classes=mixdb['num_classes'],
+                          truth_mutex=mixdb['truth_mutex'])
 
     feature = to_numpy_array(mp_feature, dtype=np.single)
-    feature = feature.reshape((genft_global['feature_frames'], fg.stride, fg.num_bands))
+    feature = feature.reshape((feature_frames, fg.stride, fg.num_bands))
 
     truth_f = to_numpy_array(mp_truth_f, dtype=np.single)
-    truth_f = truth_f.reshape((genft_global['feature_frames'], fg.num_classes))
+    truth_f = truth_f.reshape((feature_frames, fg.num_classes))
 
     segsnr = to_numpy_array(mp_segsnr, dtype=np.single)
 
-    record = genft_global['mixdb']['mixtures'][mixid]
-    first_i_frame_offset = genft_global['mixdb']['mixtures'][0]['i_frame_offset']
-    transform_frames = get_transform_frames_in_mixture(genft_global['mixdb'], mixid)
+    record = mixdb['mixtures'][mixid]
+    first_i_frame_offset = mixdb['mixtures'][0]['i_frame_offset']
+    transform_frames = get_transform_frames_in_mixture(mixdb, mixid)
 
     frame_indices = slice(record['i_frame_offset'] - first_i_frame_offset,
                           record['i_frame_offset'] - first_i_frame_offset + transform_frames)
@@ -202,22 +205,22 @@ def _process_mixture(mixid: int) -> None:
      truth_t,
      _,
      _,
-     segsnr[frame_indices]) = get_audio_and_truth_t(mixdb=genft_global['mixdb'],
+     segsnr[frame_indices]) = get_audio_and_truth_t(mixdb=mixdb,
                                                     record=record,
-                                                    target_audios=genft_global['target_audios'],
-                                                    noise_audios=genft_global['noise_audios'],
+                                                    target_audios=target_audios,
+                                                    noise_audios=noise_audios,
                                                     compute_truth=True,
-                                                    compute_segsnr=genft_global['compute_segsnr'],
+                                                    compute_segsnr=compute_segsnr,
                                                     frame_based_segnsr=True)
 
-    first_o_frame_offset = genft_global['mixdb']['mixtures'][0]['o_frame_offset']
+    first_o_frame_offset = mixdb['mixtures'][0]['o_frame_offset']
     feature_frame = record['o_frame_offset'] - first_o_frame_offset
     for transform_frame in range(transform_frames):
-        indices = slice(transform_frame * genft_global['mixdb']['frame_size'],
-                        (transform_frame + 1) * genft_global['mixdb']['frame_size'])
+        indices = slice(transform_frame * mixdb['frame_size'],
+                        (transform_frame + 1) * mixdb['frame_size'])
         mixture_fd = fft.execute(int16_to_float(mixture_td[indices]))
 
-        fg.execute(mixture_fd, truth_reduction(truth_t[indices], genft_global['mixdb']['truth_reduction_function']))
+        fg.execute(mixture_fd, truth_reduction(truth_t[indices], mixdb['truth_reduction_function']))
 
         if fg.eof():
             feature[feature_frame] = fg.feature()
