@@ -32,6 +32,7 @@ import ctypes
 import json
 import multiprocessing
 import time
+from functools import partial
 from os.path import splitext
 from typing import List
 from typing import Union
@@ -66,8 +67,6 @@ from sonusai.utils import seconds_to_hms
 from sonusai.utils import to_numpy_array
 from sonusai.utils import trim_docstring
 
-genmix_global = dict()
-
 
 # noinspection PyGlobalUndefined
 
@@ -115,36 +114,38 @@ def genmix(mixdb: dict,
            initial_i_frame_offset: int = 0,
            initial_o_frame_offset: int = 0) -> (np.ndarray, np.ndarray, np.ndarray, np.ndarray, dict):
     mixdb_out, total_samples = init(mixdb=mixdb, mixid=mixid, logging=logging)
-
-    genmix_global['mixdb'] = mixdb_out
-    genmix_global['total_samples'] = total_samples
-    genmix_global['compute_truth'] = compute_truth
-    genmix_global['compute_segsnr'] = compute_segsnr
-    genmix_global['noise_audios'] = build_noise_audio_db(mixdb_out)
-    genmix_global['target_audios'] = build_target_audio_db(mixdb_out)
+    compute_truth = compute_truth
+    compute_segsnr = compute_segsnr
+    target_audios = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+    noise_audios = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
 
     mp_mixture = multiprocessing.Array(ctypes.c_int16, total_samples)
-    mp_truth_t = multiprocessing.Array(ctypes.c_float, 0)
     if compute_truth:
         mp_truth_t = multiprocessing.Array(ctypes.c_float, total_samples * mixdb_out['num_classes'])
+    else:
+        mp_truth_t = multiprocessing.Array(ctypes.c_float, 0)
     mp_target = multiprocessing.Array(ctypes.c_int16, total_samples)
     mp_noise = multiprocessing.Array(ctypes.c_int16, total_samples)
-    mp_segsnr = multiprocessing.Array(ctypes.c_float, 0)
     if compute_segsnr:
         mp_segsnr = multiprocessing.Array(ctypes.c_float, total_samples)
+    else:
+        mp_segsnr = multiprocessing.Array(ctypes.c_float, 0)
 
     mixture = to_numpy_array(mp_mixture, dtype=np.int16)
-    truth_t = np.empty(0, dtype=np.single)
     if compute_truth:
         truth_t = to_numpy_array(mp_truth_t, dtype=np.single).reshape((total_samples, mixdb_out['num_classes']))
+    else:
+        truth_t = np.empty(0, dtype=np.single)
+
     target = to_numpy_array(mp_target, dtype=np.int16)
     noise = to_numpy_array(mp_noise, dtype=np.int16)
-    segsnr = np.empty(0, dtype=np.single)
     if compute_segsnr:
         segsnr = to_numpy_array(mp_segsnr, dtype=np.single)
+    else:
+        segsnr = np.empty(0, dtype=np.single)
 
     # First pass to get offsets
-    set_mixture_offsets(mixdb_out,
+    set_mixture_offsets(mixdb=mixdb_out,
                         initial_i_sample_offset=initial_i_sample_offset,
                         initial_i_frame_offset=initial_i_frame_offset,
                         initial_o_frame_offset=initial_o_frame_offset)
@@ -155,11 +156,17 @@ def genmix(mixdb: dict,
         progress = tqdm(total=len(mixdb_out['mixtures']), desc='genmix', disable=not show_progress)
         progress_needs_close = True
 
-    p_map(_process_mixture, mixdb_out['mixtures'], progress=progress, initializer=mp_init, initargs=(mp_mixture,
-                                                                                                     mp_truth_t,
-                                                                                                     mp_target,
-                                                                                                     mp_noise,
-                                                                                                     mp_segsnr))
+    p_map(partial(_process_mixture,
+                  mixdb=mixdb_out,
+                  total_samples=total_samples,
+                  target_audios=target_audios,
+                  noise_audios=noise_audios,
+                  compute_truth=compute_truth,
+                  compute_segsnr=compute_segsnr),
+          list(range(len(mixdb_out['mixtures']))),
+          progress=progress,
+          initializer=mp_init,
+          initargs=(mp_mixture, mp_truth_t, mp_target, mp_noise, mp_segsnr))
 
     if progress_needs_close:
         progress.close()
@@ -179,26 +186,41 @@ def genmix(mixdb: dict,
     return mixture, truth_t, target, noise, segsnr, mixdb_out
 
 
-def _process_mixture(record: dict) -> None:
-    indices = slice(record['i_sample_offset'], record['i_sample_offset'] + record['samples'])
+def _process_mixture(mixid: int,
+                     mixdb: dict,
+                     total_samples: int,
+                     target_audios: list,
+                     noise_audios: list,
+                     compute_truth: bool,
+                     compute_segsnr: bool) -> None:
     mixture = to_numpy_array(mp_mixture, dtype=np.int16)
+
     truth_t = to_numpy_array(mp_truth_t, dtype=np.single)
-    if genmix_global['compute_truth']:
-        truth_t = truth_t.reshape((genmix_global['total_samples'], genmix_global['mixdb']['num_classes']))
+    truth_t = truth_t.reshape((total_samples, mixdb['num_classes']))
+
     target = to_numpy_array(mp_target, dtype=np.int16)
+
     noise = to_numpy_array(mp_noise, dtype=np.int16)
+
     segsnr = to_numpy_array(mp_segsnr, dtype=np.single)
 
+    record = mixdb['mixtures'][mixid]
+    first_i_sample_offset = mixdb['mixtures'][0]['i_sample_offset']
+    samples = get_samples_in_mixture(mixdb, mixid)
+
+    indices = slice(record['i_sample_offset'] - first_i_sample_offset,
+                    record['i_sample_offset'] - first_i_sample_offset + samples)
+
     (mixture[indices],
      truth_t[indices],
      target[indices],
      noise[indices],
-     segsnr[indices]) = get_audio_and_truth_t(mixdb=genmix_global['mixdb'],
+     segsnr[indices]) = get_audio_and_truth_t(mixdb=mixdb,
                                               record=record,
-                                              target_audios=genmix_global['target_audios'],
-                                              noise_audios=genmix_global['noise_audios'],
-                                              compute_truth=genmix_global['compute_truth'],
-                                              compute_segsnr=genmix_global['compute_segsnr'])
+                                              target_audios=target_audios,
+                                              noise_audios=noise_audios,
+                                              compute_truth=compute_truth,
+                                              compute_segsnr=compute_segsnr)
 
 
 def main():
@@ -232,23 +254,24 @@ def main():
         check_audio_files_exist(mixdb)
 
         with h5py.File(output_name, 'w') as f:
-            mixture_dataset = f.create_dataset(name='mixture',
-                                               shape=(total_samples,),
-                                               dtype=np.int16)
+            f.create_dataset(name='mixture',
+                             shape=(total_samples,),
+                             dtype=np.int16)
             if compute_truth:
-                truth_dataset = f.create_dataset(name='truth_t',
-                                                 shape=(total_samples, mixdb_out['num_classes']),
-                                                 dtype=np.single)
-            target_dataset = f.create_dataset(name='target',
-                                              shape=(total_samples,),
-                                              dtype=np.int16)
-            noise_dataset = f.create_dataset(name='noise',
-                                             shape=(total_samples,),
-                                             dtype=np.int16)
+                f.create_dataset(name='truth_t',
+                                 shape=(total_samples, mixdb_out['num_classes']),
+                                 dtype=np.single)
+            f.create_dataset(name='target',
+                             shape=(total_samples,),
+                             dtype=np.int16)
+            f.create_dataset(name='noise',
+                             shape=(total_samples,),
+                             dtype=np.int16)
             if compute_segsnr:
-                segsnr_dataset = f.create_dataset(name='segsnr',
-                                                  shape=(total_samples,),
-                                                  dtype=np.single)
+                f.create_dataset(name='segsnr',
+                                 shape=(total_samples,),
+                                 dtype=np.single)
+
         chunk_size = 100
         progress = tqdm(total=len(mixid), desc='genmix')
         mixid = grouper(range(len(mixdb_out['mixtures'])), chunk_size)
