@@ -46,8 +46,13 @@ where mix_gfr32ts2.yml contains:
 targets:
   - data/target.wav
 
-noises:
-  - data/noise.wav
+noise_mix:
+  files:
+    - data/noise.wav
+  augmentations:
+    - normalize: -3.5
+  snrs:
+    - 20
 
 target_count: 5
 
@@ -56,15 +61,9 @@ feature: gfr32ts2
 output: data/my_mix.h5
 
 target_augmentations:
-  -
-    normalize: -3.5
+  - normalize: -3.5
     pitch: [-3, 0, 3]
     tempo: [0.8, 1, 1.2]
-    snr: 20
-
-noise_augmentations:
-  -
-    normalize: -3.5
 ...
 
 In this example a time-domain mixture is created and feature data is calculated as
@@ -117,11 +116,9 @@ will find all .wav files in the specified directories and process them as target
 
 """
 import json
-import re
 import time
-from copy import deepcopy
+from functools import partial
 from glob import glob
-from numbers import Number
 from os import listdir
 from os.path import dirname
 from os.path import isabs
@@ -129,7 +126,8 @@ from os.path import isdir
 from os.path import join
 from os.path import splitext
 from random import seed
-from random import uniform
+from typing import List
+from typing import Union
 
 import numpy as np
 import sox
@@ -145,13 +143,15 @@ from sonusai import initial_log_messages
 from sonusai import logger
 from sonusai import update_console_handler
 from sonusai.mixture import apply_augmentation
+from sonusai.mixture import balance_classes
 from sonusai.mixture import estimate_audio_length
-from sonusai.mixture import generate_truth
+from sonusai.mixture import get_augmentations
 from sonusai.mixture import get_class_weights_threshold
-from sonusai.mixture import get_config_from_file
 from sonusai.mixture import get_next_noise
 from sonusai.mixture import get_total_class_count
 from sonusai.mixture import load_config
+from sonusai.mixture import process_target
+from sonusai.mixture import raw_load_config
 from sonusai.mixture import read_audio
 from sonusai.mixture import update_truth_settings
 from sonusai.utils import expandvars
@@ -160,8 +160,6 @@ from sonusai.utils import p_map
 from sonusai.utils import seconds_to_hms
 from sonusai.utils import trim_docstring
 
-genmixdb_global = dict()
-
 
 def genmixdb(file: str = None,
              config: dict = None,
@@ -171,7 +169,7 @@ def genmixdb(file: str = None,
         raise SonusAIError(f'Must specify either file name or config.')
 
     if file is not None:
-        config = get_config_from_file(file)
+        config = load_config(file)
 
     seed(config['seed'])
 
@@ -192,11 +190,7 @@ def genmixdb(file: str = None,
 
     if logging:
         logger.info('Collecting noises')
-    noises = get_input_files(records=config['noises'])
-    if len(noises) == 0:
-        if logging:
-            logger.warning(f'Did not find any noises; using default noise: {sonusai.mixture.DEFAULT_NOISE}')
-        noises = get_input_files(records=[sonusai.mixture.DEFAULT_NOISE])
+    noises = get_input_files(records=config['noise_mix']['files'])
     if logging:
         logger.debug('List of noises:')
         logger.debug(yaml.dump([sub['name'] for sub in noises], default_flow_style=False))
@@ -213,7 +207,7 @@ def genmixdb(file: str = None,
 
     if logging:
         logger.info('Collecting noise augmentations')
-    noise_augmentations = get_augmentations(config['noise_augmentations'], target=False)
+    noise_augmentations = get_augmentations(config['noise_mix']['augmentations'], target=False)
     expanded_noise_augmentations = ''
     for augmentation in noise_augmentations:
         expanded_noise_augmentations += f'- {augmentation}\n'
@@ -221,13 +215,6 @@ def genmixdb(file: str = None,
         logger.debug('Expanded list of noise augmentations:')
         logger.debug(expanded_noise_augmentations)
 
-    noise_sets = len(noises) * len(noise_augmentations)
-    target_sets = len(targets) * len(target_augmentations)
-    total_mixtures = noise_sets * target_sets
-    if logging:
-        logger.info('')
-        logger.info(f'Found {total_mixtures} mixtures to process')
-
     class_weights_threshold = get_class_weights_threshold(config)
 
     if config['truth_mode'] not in ['normal', 'mutex']:
@@ -255,15 +242,96 @@ def genmixdb(file: str = None,
     feature_samples = config['frame_size'] * decimation * stride
     feature_step_samples = config['frame_size'] * decimation * step
 
+    mixdb = {
+        'class_count':              list(),
+        'class_labels':             config['class_labels'],
+        'class_weights_threshold':  list(class_weights_threshold),
+        'feature':                  config['feature'],
+        'feature_samples':          feature_samples,
+        'feature_step_samples':     feature_step_samples,
+        'frame_size':               config['frame_size'],
+        'mixtures':                 list(),
+        'noise_mix':                {
+            'files':         noises,
+            'augmentations': noise_augmentations,
+            'snrs':          config['noise_mix']['snrs'],
+            'exhaustive':    config['noise_mix']['exhaustive'],
+        },
+        'num_classes':              config['num_classes'],
+        'target_augmentations':     target_augmentations,
+        'targets':                  targets,
+        'truth_mutex':              truth_mutex,
+        'truth_reduction_function': config['truth_reduction_function'],
+    }
+
+    # Read in all audio data beforehand to avoid reading it multiple times in the loop
+    if logging:
+        logger.debug('Reading noise audio')
+    raw_noise_audio = list()
+    for noise in noises:
+        raw_noise_audio.append(read_audio(name=noise['name']))
+
+    if logging:
+        logger.debug('Reading target audio')
+    raw_target_audio = list()
+    for target in targets:
+        raw_target_audio.append(read_audio(name=target['name']))
+
+    # Apply all target augmentations to all targets and generate initial class count
+    augmented_targets = []
+    for target_index, target in enumerate(targets):
+        for target_augmentation_index in range(len(target_augmentations)):
+            if not isinstance(target['truth_settings'], list):
+                target['truth_settings'] = [target['truth_settings']]
+
+            update_truth_settings(target['truth_settings'], config['truth_settings'])
+
+            for truth_setting in target['truth_settings']:
+                if not isinstance(truth_setting['index'], list):
+                    truth_setting['index'] = [truth_setting['index']]
+
+                if any(idx > max_class for idx in truth_setting['index']):
+                    raise SonusAIError('invalid truth index')
+
+            augmented_targets.append({
+                'target_file_index':         target_index,
+                'target_augmentation_index': target_augmentation_index,
+            })
+
+    progress = tqdm(total=len(augmented_targets), desc='genmixdb 1', disable=not show_progress)
+
+    augmented_targets = p_map(partial(process_target,
+                                      mixdb=mixdb,
+                                      raw_target_audio=raw_target_audio),
+                              augmented_targets,
+                              progress=progress)
+
+    progress.close()
+
+    # Balance class data
+    if config['class_balancing']:
+        augmented_targets = balance_classes(mixdb=mixdb,
+                                            augmented_targets=augmented_targets,
+                                            class_balancing_augmentation=config['class_balancing_augmentation'],
+                                            raw_target_audio=raw_target_audio)
+
+    noise_sets = len(noises) * len(noise_augmentations)
+    target_sets = len(augmented_targets) * len(config['noise_mix']['snrs'])
+    total_mixtures = noise_sets * target_sets
+    if logging:
+        logger.info('')
+        logger.info(f'Found {total_mixtures} mixtures to process')
+
     total_duration = 0
-    for duration in [sub['duration'] for sub in targets]:
-        for augmentation in target_augmentations:
-            length = int(duration * sonusai.mixture.SAMPLE_RATE)
-            if 'tempo' in augmentation:
-                length /= augmentation['tempo']
-            if length % feature_step_samples:
-                length += feature_step_samples - int(length % feature_step_samples)
-            total_duration += float(length) / sonusai.mixture.SAMPLE_RATE
+    for augmented_target in augmented_targets:
+        length = int(targets[augmented_target['target_file_index']]['duration'] * sonusai.mixture.SAMPLE_RATE)
+        augmentation = target_augmentations[augmented_target['target_augmentation_index']]
+        if 'tempo' in augmentation:
+            length /= augmentation['tempo']
+        if length % feature_step_samples:
+            length += feature_step_samples - int(length % feature_step_samples)
+        total_duration += float(length) / sonusai.mixture.SAMPLE_RATE
+    total_duration *= len(config['noise_mix']['snrs'])
     total_duration *= noise_sets
 
     total_samples = total_duration * sonusai.mixture.SAMPLE_RATE
@@ -285,38 +353,7 @@ def genmixdb(file: str = None,
         logger.info(f'Feature samples:      {feature_samples} samples ({feature_ms} ms)')
         logger.info(f'Feature step samples: {feature_step_samples} samples ({feature_step_ms} ms)')
 
-    mixdb = {
-        'class_count':              list(),
-        'class_labels':             config['class_labels'],
-        'class_weights_threshold':  list(class_weights_threshold),
-        'feature':                  config['feature'],
-        'feature_samples':          feature_samples,
-        'feature_step_samples':     feature_step_samples,
-        'frame_size':               config['frame_size'],
-        'mixtures':                 list(),
-        'noise_augmentations':      noise_augmentations,
-        'noises':                   noises,
-        'num_classes':              config['num_classes'],
-        'target_augmentations':     target_augmentations,
-        'targets':                  targets,
-        'truth_mutex':              truth_mutex,
-        'truth_reduction_function': config['truth_reduction_function'],
-    }
-
-    # Read in all audio data beforehand to avoid reading it multiple times in the loop
-    if logging:
-        logger.debug('Reading noise audio')
-    raw_noise_audio = list()
-    for noise in noises:
-        raw_noise_audio.append(read_audio(name=noise['name']))
-
-    if logging:
-        logger.debug('Reading target audio')
-    raw_target_audio = list()
-    for target in targets:
-        raw_target_audio.append(read_audio(name=target['name']))
-
-    # First pass to get indices and offsets
+    # Get indices and offsets
     mixtures = [[] for _ in range(noise_sets)]
     n_id = 0
     for noise_index, noise in enumerate(noises):
@@ -327,25 +364,25 @@ def genmixdb(file: str = None,
             noise_length = estimate_audio_length(audio_in=raw_noise_audio[noise_index],
                                                  augmentation=noise_augmentation,
                                                  length_common_denominator=1)
-            for target_index, target in enumerate(targets):
-                for target_augmentation_index, target_augmentation in enumerate(target_augmentations):
-                    if not isinstance(target['truth_settings'], list):
-                        target['truth_settings'] = [target['truth_settings']]
+            for target in augmented_targets:
+                target_index = target['target_file_index']
+                target_augmentation_index = target['target_augmentation_index']
+                target_augmentation = mixdb['target_augmentations'][target_augmentation_index]
 
-                    update_truth_settings(target['truth_settings'], config['truth_settings'])
-
-                    for truth_setting in target['truth_settings']:
-                        if not isinstance(truth_setting['index'], list):
-                            truth_setting['index'] = [truth_setting['index']]
-
-                        if any(idx > max_class for idx in truth_setting['index']):
-                            raise SonusAIError('invalid truth index')
-
-                    mixtures[n_id][t_id] = {'target_file_index':         target_index,
-                                            'noise_file_index':          noise_index,
-                                            'noise_offset':              noise_offset,
-                                            'target_augmentation_index': target_augmentation_index,
-                                            'noise_augmentation_index':  noise_augmentation_index}
+                for snr in config['noise_mix']['snrs']:
+                    mixtures[n_id][t_id] = {
+                        'target_file_index':         target_index,
+                        'noise_file_index':          noise_index,
+                        'noise_offset':              noise_offset,
+                        'target_augmentation_index': target_augmentation_index,
+                        'noise_augmentation_index':  noise_augmentation_index,
+                        'snr':                       snr,
+                        'target_snr_gain':           np.NaN,
+                        'noise_snr_gain':            np.NaN,
+                        'samples':                   target['samples'],
+                        'target_gain':               target['target_gain'],
+                        'class_count':               target['class_count'],
+                    }
                     t_id += 1
 
                     target_length = estimate_audio_length(audio_in=raw_target_audio[target_index],
@@ -355,21 +392,20 @@ def genmixdb(file: str = None,
 
             n_id += 1
 
-    genmixdb_global['mixdb'] = mixdb
-    genmixdb_global['config'] = config
-    genmixdb_global['target_augmentations'] = target_augmentations
-    genmixdb_global['raw_target_audio'] = raw_target_audio
-    genmixdb_global['feature_step_samples'] = feature_step_samples
+    progress = tqdm(total=total_mixtures, desc='genmixdb 2', disable=not show_progress)
 
-    progress = tqdm(total=total_mixtures, desc='genmixdb', disable=not show_progress)
-
-    # Second pass to fill in the details
+    # Fill in the details
     for n_id in range(len(mixtures)):
-        genmixdb_global['augmented_noise_audio'] = apply_augmentation(
+        augmented_noise_audio = apply_augmentation(
             audio_in=raw_noise_audio[mixtures[n_id][0]['noise_file_index']],
             augmentation=noise_augmentations[mixtures[n_id][0]['noise_augmentation_index']],
             length_common_denominator=1)
-        mixtures[n_id] = p_map(_process_mixture, mixtures[n_id], progress=progress)
+        mixtures[n_id] = p_map(partial(_process_mixture,
+                                       mixdb=mixdb,
+                                       raw_target_audio=raw_target_audio,
+                                       augmented_noise_audio=augmented_noise_audio),
+                               mixtures[n_id],
+                               progress=progress)
 
     progress.close()
 
@@ -395,7 +431,8 @@ def genmixdb(file: str = None,
     return mixdb
 
 
-def get_input_files(records, truth_settings=None):
+def get_input_files(records: List[dict],
+                    truth_settings: Union[List[dict], None] = None) -> List[dict]:
     if truth_settings is None:
         truth_settings = list()
 
@@ -405,10 +442,12 @@ def get_input_files(records, truth_settings=None):
     return files
 
 
-def append_input_files(files: list, in_record, truth_settings: list, tokens=None):
+def append_input_files(files: List[dict],
+                       in_record: Union[dict, str],
+                       truth_settings: List[dict],
+                       tokens: Union[dict, None] = None) -> None:
     if tokens is None:
         tokens = dict()
-    in_name = in_record
 
     if isinstance(in_record, dict):
         if 'target_name' in in_record:
@@ -418,6 +457,8 @@ def append_input_files(files: list, in_record, truth_settings: list, tokens=None
 
         if 'truth_settings' in in_record:
             truth_settings = in_record['truth_settings']
+    else:
+        in_name = in_record
 
     in_name, new_tokens = expandvars(in_name)
     tokens.update(new_tokens)
@@ -449,7 +490,7 @@ def append_input_files(files: list, in_record, truth_settings: list, tokens=None
                                 append_input_files(files, child, truth_settings, tokens)
                 elif ext == '.yml':
                     try:
-                        yml_config = load_config(name)
+                        yml_config = raw_load_config(name)
 
                         if 'targets' in yml_config:
                             for record in yml_config['targets']:
@@ -477,111 +518,6 @@ def append_input_files(files: list, in_record, truth_settings: list, tokens=None
                 raise SonusAIError(f'Error processing {name}: {e}')
 
 
-def get_augmentations(rules: list, target: bool) -> list:
-    augmentations = list()
-    for rule in rules:
-        expand_augmentations(augmentations, rule, target)
-
-    augmentations = rand_augmentations(augmentations)
-    return augmentations
-
-
-def expand_augmentations(augmentations: list, rule: dict, target: bool):
-    # replace old 'eq' rule with new 'eq1' rule to allow both for backward compatibility
-    rule = {'eq1' if key == 'eq' else key: value for key, value in rule.items()}
-
-    for key in rule:
-        if key not in sonusai.mixture.VALID_AUGMENTATIONS:
-            raise SonusAIError(f'Invalid augmentation: {key}')
-
-        if key in ['eq1', 'eq2', 'eq3']:
-            # eq must be a list of length 3 or a list of length 3 lists
-            valid = True
-            multiple = False
-            if isinstance(rule[key], list):
-                if any(isinstance(el, list) for el in rule[key]):
-                    multiple = True
-                    for value in rule[key]:
-                        if not isinstance(value, list) or len(value) != 3:
-                            valid = False
-                else:
-                    if len(rule[key]) != 3:
-                        valid = False
-            else:
-                valid = False
-
-            if not valid:
-                raise SonusAIError(f'Invalid augmentation value for {key}: {rule[key]}')
-
-            if multiple:
-                for value in rule[key]:
-                    expanded_rule = deepcopy(rule)
-                    expanded_rule[key] = deepcopy(value)
-                    expand_augmentations(augmentations, expanded_rule, target)
-                return
-
-        elif key == 'count':
-            pass
-
-        else:
-            if isinstance(rule[key], list):
-                for value in rule[key]:
-                    if isinstance(value, list):
-                        raise SonusAIError(f'Invalid augmentation value for {key}: {rule[key]}')
-                    expanded_rule = deepcopy(rule)
-                    expanded_rule[key] = deepcopy(value)
-                    expand_augmentations(augmentations, expanded_rule, target)
-                return
-            elif not isinstance(rule[key], Number):
-                if not rule[key].startswith('rand'):
-                    raise SonusAIError(f'Invalid augmentation value for {key}: {rule[key]}')
-
-    # Set default snr for target if needed
-    if target and 'snr' not in rule:
-        rule['snr'] = sonusai.mixture.DEFAULT_SNR
-
-    # Remove snr rule from noise if needed
-    if not target and 'snr' in rule:
-        del rule['snr']
-
-    augmentations.append(rule)
-
-
-def rand_repl(m):
-    return f'{uniform(float(m.group(1)), float(m.group(4))):.2f}'
-
-
-def rand_augmentations(in_rules):
-    out_rules = list()
-    for rule in in_rules:
-        # do any keys contain rand?
-        has_rand = False
-        for key in rule:
-            if 'rand' in str(rule[key]):
-                has_rand = True
-                break
-
-        if has_rand:
-            count = 1
-            if rule['count'] is not None:
-                count = rule['count']
-                del rule['count']
-            for i in range(count):
-                new_rule = deepcopy(rule)
-                for key in new_rule:
-                    new_rule[key] = eval(re.sub(sonusai.mixture.RAND_PATTERN, rand_repl, str(new_rule[key])))
-
-                    # convert eq values from strings to numbers
-                    if key in ['eq1', 'eq2', 'eq3']:
-                        for n in range(3):
-                            if isinstance(new_rule[key][n], str):
-                                new_rule[key][n] = eval(new_rule[key][n])
-                out_rules.append(new_rule)
-        else:
-            out_rules.append(rule)
-    return out_rules
-
-
 def get_output_from_config(config: dict, config_name: str) -> str:
     try:
         config_base = splitext(config_name)[0]
@@ -592,74 +528,64 @@ def get_output_from_config(config: dict, config_name: str) -> str:
         raise SonusAIError(f'Error getting genmixdb base name: {e}')
 
 
-def _process_mixture(mixture: dict) -> dict:
-    target_index = mixture['target_file_index']
-    noise_offset = mixture['noise_offset']
-    target_augmentation_index = mixture['target_augmentation_index']
+def _process_mixture(record: dict,
+                     mixdb: dict,
+                     raw_target_audio: list,
+                     augmented_noise_audio: np.ndarray) -> dict:
+    target_index = record['target_file_index']
+    noise_offset = record['noise_offset']
+    target_augmentation_index = record['target_augmentation_index']
 
-    target_augmentation = genmixdb_global['target_augmentations'][target_augmentation_index]
+    target_augmentation = mixdb['target_augmentations'][target_augmentation_index]
 
-    augmented_target_audio = apply_augmentation(audio_in=genmixdb_global['raw_target_audio'][target_index],
+    augmented_target_audio = apply_augmentation(audio_in=raw_target_audio[target_index],
                                                 augmentation=target_augmentation,
-                                                length_common_denominator=genmixdb_global['feature_step_samples'])
+                                                length_common_denominator=mixdb['feature_step_samples'])
 
     noise_segment, _ = get_next_noise(offset_in=noise_offset,
                                       length=len(augmented_target_audio),
-                                      audio_in=genmixdb_global['augmented_noise_audio'])
+                                      audio_in=augmented_noise_audio)
 
-    # target_gain is the sum of 'gain', noise overflow adjustment, and mixture
-    # overflow adjustment. It is used to return the target audio to its normalized
-    # level when calculating truth (unless target gain is set to zero, in which
-    # case the truth will be all zeros).
-    if 'gain' in target_augmentation:
-        target_gain = 10 ** (target_augmentation['gain'] / 20)
+    if len(augmented_target_audio) != record['samples']:
+        raise SonusAIError('Number of samples in mixture does not match initial value')
+
+    if record['snr'] < -96:
+        # Special case for zeroing out target data
+        record['target_snr_gain'] = 0
+        record['noise_snr_gain'] = 1
+        record['class_count'] = [[0] * len(inner) for inner in record['class_count']]
+        # Setting target_gain to zero will cause the truth to be all zeros.
+        record['target_gain'] = 0
+    elif record['snr'] > 96:
+        # Special case for zeroing out noise data
+        record['target_snr_gain'] = 1
+        record['noise_snr_gain'] = 0
     else:
-        target_gain = 1
+        target_energy = np.mean(np.square(np.single(augmented_target_audio)))
+        noise_energy = np.mean(np.square(np.single(noise_segment)))
+        noise_gain = np.sqrt(target_energy / noise_energy) / 10 ** (
+                record['snr'] / 20)
 
-    if 'snr' in target_augmentation:
-        if target_augmentation['snr'] < -96:
-            # Special case for zeroing out target data
-            mixture['target_snr_gain'] = 0
-            mixture['noise_snr_gain'] = 1
-            target_gain = 0
+        # Check for noise_gain > 1 to avoid clipping
+        if noise_gain > 1:
+            record['target_snr_gain'] = 1 / noise_gain
+            record['noise_snr_gain'] = 1
         else:
-            target_energy = np.mean(np.square(np.single(augmented_target_audio)))
-            noise_energy = np.mean(np.square(np.single(noise_segment)))
-            noise_gain = np.sqrt(target_energy / noise_energy) / 10 ** (
-                    target_augmentation['snr'] / 20)
-
-            # Check for noise_gain > 1 to avoid clipping
-            if noise_gain > 1:
-                mixture['target_snr_gain'] = 1 / noise_gain
-                mixture['noise_snr_gain'] = 1
-                target_gain = target_gain / noise_gain
-            else:
-                mixture['target_snr_gain'] = 1
-                mixture['noise_snr_gain'] = noise_gain
+            record['target_snr_gain'] = 1
+            record['noise_snr_gain'] = noise_gain
 
     # Check for clipping in mixture
-    gain_adjusted_target_audio = np.single(augmented_target_audio) * mixture['target_snr_gain']
-    gain_adjusted_noise_audio = np.single(noise_segment) * mixture['noise_snr_gain']
+    gain_adjusted_target_audio = np.single(augmented_target_audio) * record['target_snr_gain']
+    gain_adjusted_noise_audio = np.single(noise_segment) * record['noise_snr_gain']
     mixture_audio = gain_adjusted_target_audio + gain_adjusted_noise_audio
 
     if any(abs(mixture_audio) >= 32768):
         # Clipping occurred; lower gains to bring audio within int16 bounds
         gain_adjustment = 32760 / max(abs(mixture_audio))
-        mixture['target_snr_gain'] *= gain_adjustment
-        mixture['noise_snr_gain'] *= gain_adjustment
-        target_gain *= gain_adjustment
+        record['target_snr_gain'] *= gain_adjustment
+        record['noise_snr_gain'] *= gain_adjustment
 
-    mixture['samples'] = len(augmented_target_audio)
-    mixture['target_gain'] = target_gain
-
-    gain_adjusted_target_audio = np.int16(np.single(augmented_target_audio) * mixture['target_snr_gain'])
-    gain_adjusted_noise_audio = np.int16(np.single(noise_segment) * mixture['noise_snr_gain'])
-    generate_truth(mixdb=genmixdb_global['mixdb'],
-                   record=mixture,
-                   target=gain_adjusted_target_audio,
-                   noise=gain_adjusted_noise_audio)
-
-    return mixture
+    return record
 
 
 def main():
@@ -671,7 +597,7 @@ def main():
         for config_file in args['CONFIG']:
             start_time = time.monotonic()
             logger.info(f'Creating mixture database for {config_file}')
-            config = get_config_from_file(config_file)
+            config = load_config(config_file)
             output = get_output_from_config(config, config_file)
 
             log_name = output + '.log'
