@@ -66,7 +66,7 @@ from sonusai import initial_log_messages
 from sonusai import logger
 from sonusai import update_console_handler
 from sonusai.mixture import get_default_config
-from sonusai.mixture import load_config
+from sonusai.mixture import raw_load_config
 from sonusai.mixture import update_config_from_hierarchy
 from sonusai.mixture import update_truth_settings
 from sonusai.utils import trim_docstring
@@ -130,7 +130,7 @@ def gentcst(fold: str = '*',
 
         specific_name = splitext(file['file'])[0] + '.yml'
         if exists(specific_name):
-            specific_config = load_config(specific_name)
+            specific_config = raw_load_config(specific_name)
 
             for key in specific_config:
                 if key != 'truth_settings':
@@ -138,7 +138,7 @@ def gentcst(fold: str = '*',
                 else:
                     if not isinstance(specific_config['truth_settings'], list):
                         specific_config['truth_settings'] = [specific_config['truth_settings']]
-                    update_truth_settings(truth_settings=specific_config['truth_settings'],
+                    update_truth_settings(given=specific_config['truth_settings'],
                                           default=local_config['truth_settings'])
                     local_config['truth_settings'] = specific_config['truth_settings']
 
@@ -410,7 +410,7 @@ def get_config() -> dict:
     if not exists(CONFIG_FILE):
         raise SonusAIError(f'No {CONFIG_FILE} at top level')
 
-    config = load_config(CONFIG_FILE)
+    config = raw_load_config(CONFIG_FILE)
 
     if 'feature' not in config:
         raise SonusAIError('feature not in top level config')
